@@ -11,7 +11,7 @@
   (*  Contact: dev@gim.name                                                 *)
   (**************************************************************************)
 
-(* $Id: dtools.ml 4940 2007-12-12 12:44:59Z metamorph68 $ *)
+(* $Id: dtools.ml 5102 2008-02-09 14:19:41Z metamorph68 $ *)
 
 (**
   ocaml-dtools
@@ -534,6 +534,26 @@ struct
 	  exit (-1)
     end
 
+  (** A function to reopen a file descriptor
+    * Thanks to Xavier Leroy !
+    * Ref: http://caml.inria.fr/pub/ml-archives/caml-list/2000/01/a7e3bbdfaab33603320d75dbdcd40c37.en.html
+    *)
+  let reopen_out outchan filename =
+    flush outchan;
+    let fd1 = Unix.descr_of_out_channel outchan in
+    let fd2 =
+      Unix.openfile filename [Unix.O_WRONLY; Unix.O_CREAT; Unix.O_TRUNC] 0o666 in
+    Unix.dup2 fd2 fd1;
+    Unix.close fd2
+
+  (** The same for inchan *)
+  let reopen_in inchan filename =
+    let fd1 = Unix.descr_of_in_channel inchan in
+    let fd2 =
+      Unix.openfile filename [Unix.O_WRONLY; Unix.O_CREAT; Unix.O_TRUNC] 0o666 in
+    Unix.dup2 fd2 fd1;
+    Unix.close fd2
+
   let daemonize fn =
     begin match Unix.fork () with
     | 0 ->
@@ -558,11 +578,11 @@ struct
                exit 1;
         (* chdir to / *)
         Unix.chdir "/" ;
-        (* Close usual file descriptor *)
+        (* Reopen usual file descriptor *)
         flush_all ();
-        close_in stdin;
-        close_out stdout;
-        close_out stderr;
+        reopen_in stdin "/dev/null";
+        reopen_out stdout "/dev/null";
+        reopen_out stderr "/dev/null";
         (* Main loop *)
         f () ;
         exit 0
